@@ -25,6 +25,7 @@ use ruma::{
         receipt::create_receipt,
         redact::redact_event,
         state::send_state_event,
+        tag::{create_tag, delete_tag},
         typing::create_typing_event::{Request as TypingRequest, Typing},
     },
     assign,
@@ -36,15 +37,16 @@ use ruma::{
             },
             EncryptedFile,
         },
+        tag::TagInfo,
         AnyMessageEventContent, AnyStateEventContent,
     },
-    identifiers::{EventId, UserId},
     receipt::ReceiptType,
+    EventId, UserId,
 };
 #[cfg(feature = "encryption")]
 use tracing::instrument;
 
-use crate::{room::Common, BaseRoom, Client, Result, RoomType};
+use crate::{error::HttpResult, room::Common, BaseRoom, Client, HttpError, Result, RoomType};
 
 const TYPING_NOTICE_TIMEOUT: Duration = Duration::from_secs(4);
 const TYPING_NOTICE_RESEND_TIMEOUT: Duration = Duration::from_secs(3);
@@ -159,10 +161,10 @@ impl Joined {
     ///
     /// ```no_run
     /// use std::time::Duration;
-    /// use matrix_sdk::api::r0::typing::create_typing_event::Typing;
+    /// use matrix_sdk::ruma::api::client::r0::typing::create_typing_event::Typing;
     /// # use matrix_sdk::{
     /// #     Client, SyncSettings,
-    /// #     identifiers::room_id,
+    /// #     ruma::room_id,
     /// # };
     /// # use futures::executor::block_on;
     /// # use url::Url;
@@ -349,9 +351,9 @@ impl Joined {
     /// # use matrix_sdk::{Client, SyncSettings};
     /// # use url::Url;
     /// # use futures::executor::block_on;
-    /// # use matrix_sdk::identifiers::room_id;
+    /// # use matrix_sdk::ruma::room_id;
     /// # use std::convert::TryFrom;
-    /// use matrix_sdk::events::{
+    /// use matrix_sdk::ruma::events::{
     ///     AnyMessageEventContent,
     ///     room::message::{MessageEventContent, TextMessageEventContent},
     /// };
@@ -431,7 +433,7 @@ impl Joined {
     ///
     /// ```no_run
     /// # use std::{path::PathBuf, fs::File, io::Read};
-    /// # use matrix_sdk::{Client, identifiers::room_id};
+    /// # use matrix_sdk::{Client, ruma::room_id};
     /// # use url::Url;
     /// # use mime;
     /// # use futures::executor::block_on;
@@ -461,9 +463,9 @@ impl Joined {
             #[cfg(feature = "encryption")]
             let mut reader = AttachmentEncryptor::new(reader);
             #[cfg(feature = "encryption")]
-            let content_type = mime::APPLICATION_OCTET_STREAM;
+            let content_type = &mime::APPLICATION_OCTET_STREAM;
 
-            let response = self.client.upload(&content_type, &mut reader).await?;
+            let response = self.client.upload(content_type, &mut reader).await?;
 
             #[cfg(feature = "encryption")]
             let keys: Option<Box<EncryptedFile>> = {
@@ -532,18 +534,17 @@ impl Joined {
     /// # Example
     ///
     /// ```no_run
-    /// use matrix_sdk::{
+    /// use matrix_sdk::ruma::{
     ///     events::{
     ///         AnyStateEventContent,
     ///         room::member::{MemberEventContent, MembershipState},
     ///     },
-    ///     identifiers::mxc_uri,
-    ///     assign,
+    ///     assign, mxc_uri,
     /// };
     /// # futures::executor::block_on(async {
     /// # let homeserver = url::Url::parse("http://localhost:8080").unwrap();
     /// # let mut client = matrix_sdk::Client::new(homeserver).unwrap();
-    /// # let room_id = matrix_sdk::identifiers::room_id!("!test:localhost");
+    /// # let room_id = matrix_sdk::ruma::room_id!("!test:localhost");
     ///
     /// let avatar_url = mxc_uri!("mxc://example.org/avatar");
     /// let member_event = assign!(MemberEventContent::new(MembershipState::Join), {
@@ -561,7 +562,7 @@ impl Joined {
         &self,
         content: impl Into<AnyStateEventContent>,
         state_key: &str,
-    ) -> Result<send_state_event::Response> {
+    ) -> HttpResult<send_state_event::Response> {
         let content = content.into();
         let request = send_state_event::Request::new(self.inner.room_id(), state_key, &content);
 
@@ -591,11 +592,11 @@ impl Joined {
     /// # futures::executor::block_on(async {
     /// # let homeserver = url::Url::parse("http://localhost:8080").unwrap();
     /// # let mut client = matrix_sdk::Client::new(homeserver).unwrap();
-    /// # let room_id = matrix_sdk::identifiers::room_id!("!test:localhost");
+    /// # let room_id = matrix_sdk::ruma::room_id!("!test:localhost");
     /// # let room = client
     /// #   .get_joined_room(&room_id)
     /// #   .unwrap();
-    /// let event_id = matrix_sdk::identifiers::event_id!("$xxxxxx:example.org");
+    /// let event_id = matrix_sdk::ruma::event_id!("$xxxxxx:example.org");
     /// let reason = Some("Indecent material");
     /// room.redact(&event_id, reason, None).await.unwrap();
     /// # })
@@ -605,7 +606,7 @@ impl Joined {
         event_id: &EventId,
         reason: Option<&str>,
         txn_id: Option<Uuid>,
-    ) -> Result<redact_event::Response> {
+    ) -> HttpResult<redact_event::Response> {
         let txn_id = txn_id.unwrap_or_else(Uuid::new_v4).to_string();
         let request =
             assign!(redact_event::Request::new(self.inner.room_id(), event_id, &txn_id), {
@@ -614,4 +615,48 @@ impl Joined {
 
         self.client.send(request, None).await
     }
+
+    /// Adds a tag to the room, or updates it if it already exists.
+    ///
+    /// Returns the `[create_tag::Response]` from the server.
+    ///
+    /// # Arguments
+    /// * `tag` - The tag to add or update.
+    ///
+    /// * `tag_info` - Information about the tag, generally containing the
+    ///   `order` parameter.
+    ///
+    /// # Example
+    ///
+    /// ```no_run
+    /// # use ruma::events::tag::TagInfo;
+    /// # futures::executor::block_on(async {
+    /// # let homeserver = url::Url::parse("http://localhost:8080").unwrap();
+    /// # let mut client = matrix_sdk::Client::new(homeserver).unwrap();
+    /// # let room_id = matrix_sdk::ruma::room_id!("!test:localhost");
+    /// # let room = client
+    /// #   .get_joined_room(&room_id)
+    /// #   .unwrap();
+    /// let mut tag_info = TagInfo::new();
+    /// tag_info.order = Some(0.9);
+    /// room.set_tag("u.work", tag_info );
+    /// # })
+    /// ```
+    pub async fn set_tag(&self, tag: &str, tag_info: TagInfo) -> HttpResult<create_tag::Response> {
+        let user_id = self.client.user_id().await.ok_or(HttpError::AuthenticationRequired)?;
+        let request = create_tag::Request::new(&user_id, self.inner.room_id(), tag, tag_info);
+        self.client.send(request, None).await
+    }
+
+    /// Removes a tag from the room.
+    ///
+    /// Returns the `[delete_tag::Response]` from the server.
+    ///
+    /// # Arguments
+    /// * `tag` - The tag to remove.
+    pub async fn remove_tag(&self, tag: &str) -> HttpResult<delete_tag::Response> {
+        let user_id = self.client.user_id().await.ok_or(HttpError::AuthenticationRequired)?;
+        let request = delete_tag::Request::new(&user_id, self.inner.room_id(), tag);
+        self.client.send(request, None).await
+    }
 }
